@@ -26,9 +26,16 @@
 #ifdef HAVE_STDINT_H
 # include <stdint.h>
 #endif
-#include <unistd.h>
-#include <sys/types.h>
+#ifdef HAVE_UNISTD_H
+# include <unistd.h>
+#endif
+#ifdef HAVE_SYS_TYPES_H
+# include <sys/types.h>
+#endif
 #ifdef HAVE_W32_SYSTEM
+# ifdef HAVE_WINSOCK2_H
+#  include <winsock2.h>
+# endif 
 # include <windows.h>
 #else
 # include <sys/socket.h>
@@ -51,6 +58,9 @@
 #ifndef AF_LOCAL
 # define AF_LOCAL AF_UNIX
 #endif
+#ifndef INADDR_NONE
+#define INADDR_NONE  ((unsigned long)(-1))
+#endif /*INADDR_NONE*/
 
 #ifndef SUN_LEN
 # define SUN_LEN(ptr) ((size_t) (((struct sockaddr_un *) 0)->sun_path) \
@@ -87,6 +97,69 @@ parse_portno (const char *str, uint16_t *r_port)
 }
 
 
+static gpg_error_t
+_assuan_connect_finalize(assuan_context_t ctx, int fd, unsigned int flags)
+{
+  gpg_error_t err;
+
+  ctx->engine.release = _assuan_client_release;
+  ctx->engine.readfnc = _assuan_simple_read;
+  ctx->engine.writefnc = _assuan_simple_write;
+  ctx->engine.sendfd = NULL;
+  ctx->engine.receivefd = NULL;
+  ctx->finish_handler = _assuan_client_finish;
+  ctx->inbound.fd = fd;
+  ctx->outbound.fd = fd;
+  ctx->max_accepts = -1;
+
+  if (flags & ASSUAN_SOCKET_CONNECT_FDPASSING)
+    _assuan_init_uds_io (ctx);
+
+  /* initial handshake */
+  {
+    assuan_response_t response;
+    int off;
+
+    err = _assuan_read_from_server (ctx, &response, &off, 0);
+    if (err)
+      TRACE1 (ctx, ASSUAN_LOG_SYSIO, "assuan_socket_connect", ctx,
+	      "can't connect to server: %s\n", gpg_strerror (err));
+    else if (response != ASSUAN_RESPONSE_OK)
+      {
+	char *sname = _assuan_encode_c_string (ctx, ctx->inbound.line);
+	if (sname)
+	  {
+	    TRACE1 (ctx, ASSUAN_LOG_SYSIO, "assuan_socket_connect", ctx,
+		    "can't connect to server: %s", sname);
+	    _assuan_free (ctx, sname);
+	  }
+	err = _assuan_error (ctx, GPG_ERR_ASS_CONNECT_FAILED);
+      }
+  }
+
+  return err;
+}
+
+
+/* Attach an existing connected file descriptor FD to an allocated handle CTX
+ * and initialize the connection.
+ */
+gpg_error_t
+assuan_socket_connect_fd (assuan_context_t ctx, int fd, unsigned int flags)
+{
+  gpg_error_t err;
+
+  if (!ctx || fd < 0)
+    return GPG_ERR_INV_ARG;
+
+  err = _assuan_connect_finalize(ctx, fd, flags);
+
+  if (err)
+    _assuan_reset (ctx);
+
+  return err;
+}
+
 
 /* Make a connection to the Unix domain socket NAME and return a new
    Assuan context in CTX.  SERVER_PID is currently not used but may
@@ -100,13 +173,13 @@ parse_portno (const char *str, uint16_t *r_port)
 
       file://<fname>
 
-        This is the same as the defualt just with an explicit schemata.
+        This is the same as the default just with an explicit schemata.
 
       assuan://<ipaddr>:<port>
       assuan://[<ip6addr>]:<port>
          
         Connect using TCP to PORT of the server with the numerical
-        IPADDR.  Not that the '[' and ']' are literal characters.
+        IPADDR.  Note that '[' and ']' are literal characters.
 
   */
 gpg_error_t
@@ -175,7 +248,7 @@ assuan_socket_connect (assuan_context_t ctx, const char *name,
       if (!addrstr)
         return _assuan_error (ctx, gpg_err_code_from_syserror ());
 
-      if (*addrstr == '[')
+      if (*name == '[')
         {
           strcpy (addrstr, name+1);
           p = strchr (addrstr, ']');
@@ -258,41 +331,8 @@ assuan_socket_connect (assuan_context_t ctx, const char *name,
       return _assuan_error (ctx, GPG_ERR_ASS_CONNECT_FAILED);
     }
  
-  ctx->engine.release = _assuan_client_release;
-  ctx->engine.readfnc = _assuan_simple_read;
-  ctx->engine.writefnc = _assuan_simple_write;
-  ctx->engine.sendfd = NULL;
-  ctx->engine.receivefd = NULL;
-  ctx->finish_handler = _assuan_client_finish;
-  ctx->inbound.fd = fd;
-  ctx->outbound.fd = fd;
-  ctx->max_accepts = -1;
+  err = _assuan_connect_finalize(ctx, fd, flags);
 
-  if (flags & ASSUAN_SOCKET_CONNECT_FDPASSING)
-    _assuan_init_uds_io (ctx);
-
-  /* initial handshake */
-  {
-    assuan_response_t response;
-    int off;
-
-    err = _assuan_read_from_server (ctx, &response, &off);
-    if (err)
-      TRACE1 (ctx, ASSUAN_LOG_SYSIO, "assuan_socket_connect", ctx,
-	      "can't connect to server: %s\n", gpg_strerror (err));
-    else if (response != ASSUAN_RESPONSE_OK)
-      {
-	char *sname = _assuan_encode_c_string (ctx, ctx->inbound.line);
-	if (sname)
-	  {
-	    TRACE1 (ctx, ASSUAN_LOG_SYSIO, "assuan_socket_connect", ctx,
-		    "can't connect to server: %s", sname);
-	    _assuan_free (ctx, sname);
-	  }
-	err = _assuan_error (ctx, GPG_ERR_ASS_CONNECT_FAILED);
-      }
-  }
-  
   if (err)
     _assuan_reset (ctx);
 
