@@ -25,10 +25,12 @@
 #include <string.h>
 #include <stdarg.h>
 #ifdef HAVE_W32_SYSTEM
-#include <windows.h>
+# ifdef HAVE_WINSOCK2_H
+#  include <winsock2.h>
+# endif
+# include <windows.h>
 # ifdef HAVE_W32CE_SYSTEM
 # include <winioctl.h>
-# include <devload.h>
 # endif /*HAVE_W32CE_SYSTEM*/
 #endif /*HAVE_W32_SYSTEM*/
 
@@ -40,11 +42,11 @@
 const char *
 _assuan_sysutils_blurb (void)
 {
-  static const char blurb[] = 
+  static const char blurb[] =
     "\n\n"
-    "This is Libassuan - The GnuPG IPC Library\n"
+    "This is Libassuan " PACKAGE_VERSION " - The GnuPG IPC Library\n"
     "Copyright 2000, 2002, 2003, 2004, 2007, 2008, 2009,\n"
-    "          2010 Free Software Foundation, Inc.\n"
+    "          2010, 2011 Free Software Foundation, Inc.\n"
     "\n\n";
   return blurb;
 }
@@ -62,39 +64,39 @@ w32_read_registry (const wchar_t *dir, const wchar_t *name)
   DWORD n, nbytes;
   wchar_t *buffer = NULL;
   char *result = NULL;
-  
+
   if (RegOpenKeyEx (HKEY_LOCAL_MACHINE, dir, 0, KEY_READ, &handle))
     return NULL; /* No need for a RegClose, so return immediately. */
 
   nbytes = 1;
   if (RegQueryValueEx (handle, name, 0, NULL, NULL, &nbytes))
-    goto leave;
+    goto out;
   buffer = malloc ((n=nbytes+2));
   if (!buffer)
-    goto leave;
+    goto out;
   if (RegQueryValueEx (handle, name, 0, NULL, (PBYTE)buffer, &n))
     {
       free (buffer);
       buffer = NULL;
-      goto leave;
+      goto out;
     }
-  
+
   n = WideCharToMultiByte (CP_UTF8, 0, buffer, nbytes, NULL, 0, NULL, NULL);
   if (n < 0 || (n+1) <= 0)
-    goto leave;
+    goto out;
   result = malloc (n+1);
   if (!result)
-    goto leave;
+    goto out;
   n = WideCharToMultiByte (CP_UTF8, 0, buffer, nbytes, result, n, NULL, NULL);
   if (n < 0)
     {
       free (result);
       result = NULL;
-      goto leave;
+      goto out;
     }
   result[n] = 0;
 
- leave:
+ out:
   free (buffer);
   RegCloseKey (handle);
   return result;
@@ -132,4 +134,3 @@ _assuan_getenv (const char *name)
     return NULL;
 }
 #endif /*HAVE_W32CE_SYSTEM*/
-
